import logging
from logging.handlers import RotatingFileHandler

# Set root logger level to DEBUG
# 設定 root logger 等級為 DEBUG
logging.root.setLevel(logging.DEBUG)

# Create a named logger
# 建立名為 "TestPioneer" 的 logger
test_pioneer_logger = logging.getLogger("TestPioneer")

# Define log format
# 定義日誌格式
formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s')


class TestPioneerHandler(RotatingFileHandler):
    """
    Custom log handler for TestPioneer with rotating file support.
    自訂的 TestPioneer 日誌處理器，支援檔案輪替。
    """

    def __init__(self, filename: str = "TestPioneer.log", mode: str = "w",
                 max_bytes: int = 1073741824, backup_count: int = 0):
        """
        Args:
            filename (str): Log file name. 日誌檔案名稱。
            mode (str): File open mode. 檔案開啟模式。
            max_bytes (int): Max file size before rotation. 檔案輪替前的最大大小。
            backup_count (int): Number of backup files to keep. 保留的備份檔案數量。
        """
        super().__init__(filename=filename, mode=mode, maxBytes=max_bytes, backupCount=backup_count)
        self.setFormatter(formatter)  # 設定 formatter
        self.setLevel(logging.DEBUG)  # 設定等級為 DEBUG

    def emit(self, record: logging.LogRecord) -> None:
        """
        Emit a log record.
        輸出日誌紀錄。
        """
        super().emit(record)


def step_log_check(enable_logging: bool = False, logger: logging.Logger = None,
                   level: str = "info", message: str = None) -> None:
    """
    Log a message if logging is enabled.
    若啟用日誌，則記錄訊息。

    Args:
        enable_logging (bool): Whether to enable logging. 是否啟用日誌。
        logger (logging.Logger): Logger instance. Logger 實例。
        level (str): Log level ("info" or "error"). 日誌等級 ("info" 或 "error")。
        message (str): Log message. 日誌訊息。
    """
    if enable_logging and logger and message:
        logger_level = {
            "info": logger.info,
            "error": logger.error,
        }.get(level)
        if logger_level:
            logger_level(message)


template_1_str: str = \
    """
    # log file path
    pioneer_log: "test_pioneer.log"
    # test start
    jobs:
        # test steps
        steps:
            # use gui runner to run gui test
            - name: run_test_script_1
              run: test/test1.json
              with: gui-runner
            # use web runner to run gui test
            - name: run_test_script_2
              run: test/test2.json
              with: web-runner
            # use api runner to run gui test  
            - name: run_test_script_3
              run: test/test3.json
              with: api-runner
            # use load runner to run gui test
            - name: run_test_script_4
              run: test/test4.json
              with: load-runner
            # open program and redirect stdout and stderr to path
            - name: open_test_program
              open_program: test_path/test_file
              redirect_stdout: "test_std.log"
              redirect_stderr: "test_err.log"
            # stop 5 sec  s
            - name: wait_seconds
              wait: 5
            # use default browser to open url 
            - name: open_test_url
              open_url: https://www.google.com
            # close program that open use {name}
            - name: close_test_program
              close_program: open_test_program
            - name: run_folder_1
              run: /test/unit_test/run_folder/test
              with: gui-runner # you can only choose one runner to run folder
    
    """



import json
import os
from pathlib import Path

from test_pioneer.executor.run.utils import select_with_runner
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger


def run(step: dict, enable_logging: bool = False) -> bool:
    """
    Run a JSON-based task file using a selected runner.
    使用指定的 runner 執行 JSON 格式的任務檔案。

    Args:
        step (dict): Dictionary containing 'run' (file path).
                     包含 'run' (檔案路徑) 的字典。
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄。

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False。
    """

    # Select runner with validation
    # 選擇 runner 並檢查是否有效
    check_with_data = select_with_runner(step, enable_logging=enable_logging, mode="run")
    if not check_with_data[0]:
        return False
    execute_with = check_with_data[1]

    # Get file path
    # 取得檔案路徑
    file_path = step.get("run")
    if not isinstance(file_path, str):
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"'run' parameter must be a string, got: {file_path}"
        )
        return False

    # Build absolute path safely
    # 安全地建立絕對路徑
    abs_file_path = Path(os.getcwd()) / file_path
    abs_file_path = abs_file_path.resolve()

    # Validate file existence
    # 驗證檔案是否存在
    if not abs_file_path.is_file():
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"File does not exist: {file_path}"
        )
        return False

    # Load JSON content
    # 載入 JSON 內容
    try:
        file_content = json.loads(abs_file_path.read_text(encoding="utf-8"))
    except Exception as error:
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"Failed to load JSON file {file_path}, error: {error}"
        )
        return False

    # Execute with runner
    # 使用 runner 執行
    execute_with(file_content)
    return True


import shlex
import subprocess
import sys
from typing import Union, Optional

import psutil


class ExecuteProcess:
    """
    A class to start and manage external processes.
    用來啟動並管理外部子程序的類別。
    """

    def __init__(
        self,
        redirect_stdout: Union[str, int] = subprocess.PIPE,
        redirect_stderr: Union[str, int] = subprocess.PIPE,
    ):
        """
        Args:
            redirect_stdout (Union[str, int]): Path to stdout file or PIPE.
                                               標準輸出重定向檔案路徑或 PIPE。
            redirect_stderr (Union[str, int]): Path to stderr file or PIPE.
                                               標準錯誤輸出重定向檔案路徑或 PIPE。
        """
        super().__init__()
        self.process: Optional[subprocess.Popen] = None
        self.redirect_stdout: Union[str, int] = redirect_stdout
        self.redirect_stderr: Union[str, int] = redirect_stderr

    def start_process(self, shell_command: str) -> None:
        """
        Start a subprocess with given shell command.
        使用指定的 shell 命令啟動子程序。

        Args:
            shell_command (str): Command to execute. 要執行的命令字串。
        """
        # Windows 平台直接使用字串，其他平台用 shlex.split
        if sys.platform in ["win32", "cygwin", "msys"]:
            args = shell_command
        else:
            args = shlex.split(shell_command)

        # 設定 stdout 重定向
        if isinstance(self.redirect_stdout, str):
            stdout_file = open(self.redirect_stdout, "w")
        else:
            stdout_file = subprocess.PIPE

        # 設定 stderr 重定向
        if isinstance(self.redirect_stderr, str):
            stderr_file = open(self.redirect_stderr, "w")
        else:
            stderr_file = subprocess.PIPE

        # 建立子程序
        self.process = subprocess.Popen(
            args=args,
            stdout=stdout_file,
            stderr=stderr_file,
            shell=True,  # 注意：使用 shell=True 可能有安全風險
        )

    def exit_program(self) -> None:
        """
        Terminate the process and all its children.
        終止子程序及其所有子程序。
        """
        if not self.process:
            return
        try:
            process = psutil.Process(self.process.pid)
            for proc in process.children(recursive=True):
                proc.kill()
            process.kill()
        except psutil.NoSuchProcess:
            pass


import webbrowser

from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger
from test_pioneer.utils.exception.exceptions import ExecutorException


def open_url(step: dict, enable_logging: bool = False) -> bool:
    """
    Open a URL using Python's webbrowser module.
    使用 Python 的 webbrowser 模組開啟網址。

    Args:
        step (dict): A dictionary containing 'open_url' and optional 'url_open_method'.
                     包含 'open_url' 與可選的 'url_open_method' 的字典。
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄。

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False。
    """

    url = step.get("open_url")
    if not isinstance(url, str):
        # Log error if 'open_url' is not a string
        # 若 'open_url' 不是字串，記錄錯誤
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"The 'open_url' parameter is not a str type: {url}"
        )
        return False

    # Log the URL to be opened
    # 記錄即將開啟的網址
    step_log_check(
        enable_logging=enable_logging,
        logger=test_pioneer_logger,
        level="info",
        message=f"Open URL: {url}"
    )

    try:
        # Map available open methods
        # 定義可用的開啟方式
        method_map = {
            "open": webbrowser.open,
            "open_new": webbrowser.open_new,
            "open_new_tab": webbrowser.open_new_tab,
        }

        method_key = step.get("url_open_method", "open")  # 預設使用 "open"
        url_open_method = method_map.get(method_key)

        if url_open_method is None:
            # Log error if method is invalid
            # 若方法無效，記錄錯誤
            step_log_check(
                enable_logging=enable_logging,
                logger=test_pioneer_logger,
                level="error",
                message=f"Invalid url_open_method: {method_key}"
            )
            return False

        # Execute the chosen method
        # 執行對應的開啟方式
        url_open_method(url=url)

    except ExecutorException as error:
        # Log exception if occurs
        # 若發生例外，記錄錯誤
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"Failed to open URL {url}, error: {repr(error)}"
        )
        return False

    return True


from test_pioneer.logging.loggin_instance import test_pioneer_logger, step_log_check


def download_single_file(step: dict, enable_logging: bool = False) -> bool:
    """
    Download a single file using automation_file.download_file
    使用 automation_file.download_file 下載單一檔案

    Args:
        step (dict): Dictionary containing 'download_file' (URL) and 'file_path' (local path).
                     包含 'download_file' (檔案網址) 與 'file_path' (本地路徑) 的字典
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False
    """
    file_url = step.get("download_file")
    file_path = step.get("file_path")

    # Import inside function to avoid unnecessary global dependency
    # 在函式內部匯入，避免全域依賴
    from automation_file import download_file

    # Check required parameters
    # 檢查必要參數
    if not file_url or not file_path:
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message="Missing parameters: 'download_file' and 'file_path' are required."
        )
        return False

    # Validate parameter types
    # 驗證參數型別
    if not isinstance(file_url, str) or not isinstance(file_path, str):
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message="Both 'download_file' and 'file_path' must be of type str."
        )
        return False

    # Perform download
    # 執行下載
    download_file(file_url=file_url, file_name=file_path)
    return True


def unzip_zipfile(step: dict, enable_logging: bool = False) -> bool:
    """
    Unzip a zip file using automation_file.unzip_all
    使用 automation_file.unzip_all 解壓縮 zip 檔案

    Args:
        step (dict): Dictionary containing 'zip_file_path', optional 'password', and 'extract_path'.
                     包含 'zip_file_path'，可選 'password' 與 'extract_path' 的字典
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False
    """
    zip_file_path = step.get("zip_file_path")
    password = step.get("password")
    extract_path = step.get("extract_path")

    # Check required parameter
    # 檢查必要參數
    if not zip_file_path:
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message="Missing parameter: 'zip_file_path' is required."
        )
        return False

    # Validate parameter type
    # 驗證參數型別
    if not isinstance(zip_file_path, str):
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message="'zip_file_path' must be of type str."
        )
        return False

    # Import inside function to avoid unnecessary global dependency
    # 在函式內部匯入，避免全域依賴
    from automation_file import unzip_all

    # Build kwargs dynamically
    # 動態建立參數字典
    kwargs = {"zip_file_path": zip_file_path}
    if password:
        kwargs["password"] = password
    if extract_path:
        kwargs["extract_path"] = extract_path

    # Perform unzip
    # 執行解壓縮
    unzip_all(**kwargs)
    return True


import os
from pathlib import Path

from test_pioneer.executor.run.utils import select_with_runner
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger


def run_folder(step: dict, enable_logging: bool = False, mode: str = "run_folder") -> bool:
    """
    Run all JSON files inside a specified folder using a selected runner.
    使用指定的 runner 執行資料夾中的所有 JSON 檔案。

    Args:
        step (dict): Dictionary containing 'run_folder' (folder path).
                     包含 'run_folder' (資料夾路徑) 的字典。
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄。
        mode (str): Runner mode, default is "run_folder".
                    Runner 模式，預設為 "run_folder"。

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False。
    """

    # Select runner with validation
    # 選擇 runner 並檢查是否有效
    check_with_data = select_with_runner(step, enable_logging=enable_logging, mode=mode)
    if not check_with_data[0]:
        return False
    execute_with = check_with_data[1]

    # Get folder path
    # 取得資料夾路徑
    folder_path = step.get("run_folder")
    if not isinstance(folder_path, str):
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"'run_folder' parameter must be a string, got: {folder_path}"
        )
        return False

    # Build absolute path safely
    # 安全地建立絕對路徑
    abs_folder_path = Path(os.getcwd()) / folder_path
    abs_folder_path = abs_folder_path.resolve()

    # Validate folder existence
    # 驗證資料夾是否存在
    if not abs_folder_path.is_dir():
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"Folder does not exist: {folder_path}"
        )
        return False

    # Find JSON files
    # 搜尋 JSON 檔案
    json_files = list(abs_folder_path.glob("*.json"))
    if not json_files:
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"Folder is empty or contains no JSON files: {folder_path}"
        )
        return False

    # Execute runner with JSON files
    # 使用 runner 執行 JSON 檔案
    execute_with(json_files)
    return True


from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger
from test_pioneer.process.execute_process import ExecuteProcess
from test_pioneer.process.process_manager import process_manager_instance


def open_program(step: dict, name: str, enable_logging: bool = False) -> bool:
    """
    Open a program using ExecuteProcess and register it in process_manager_instance.
    使用 ExecuteProcess 開啟程式並註冊到 process_manager_instance。

    Args:
        step (dict): Dictionary containing 'open_program' and optional 'redirect_stdout', 'redirect_stderr'.
                     包含 'open_program' 以及可選的 'redirect_stdout', 'redirect_stderr' 的字典。
        name (str): The key name to register the process in process_manager_instance.
                    在 process_manager_instance 中註冊的名稱。
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄。

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False。
    """
    program = step.get("open_program")
    if not isinstance(program, str):
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"The 'open_program' parameter must be str, got: {program}"
        )
        return False

    step_log_check(
        enable_logging=enable_logging,
        logger=test_pioneer_logger,
        level="info",
        message=f"Open program: {program}"
    )

    # Initialize redirect paths
    # 初始化輸出重定向路徑
    redirect_stdout = None
    redirect_stderr = None

    # Handle stdout redirection
    # 處理標準輸出重定向
    if "redirect_stdout" in step:
        stdout_path = step.get("redirect_stdout")
        if not isinstance(stdout_path, str):
            step_log_check(
                enable_logging=enable_logging,
                logger=test_pioneer_logger,
                level="error",
                message=f"The 'redirect_stdout' parameter must be str, got: {stdout_path}"
            )
            return False
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="info",
            message=f"Redirect stdout to: {stdout_path}"
        )
        redirect_stdout = stdout_path

    # Handle stderr redirection
    # 處理標準錯誤輸出重定向
    if "redirect_stderr" in step:
        stderr_path = step.get("redirect_stderr")
        if not isinstance(stderr_path, str):
            step_log_check(
                enable_logging=enable_logging,
                logger=test_pioneer_logger,
                level="error",
                message=f"The 'redirect_stderr' parameter must be str, got: {stderr_path}"
            )
            return False
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="info",
            message=f"Redirect stderr to: {stderr_path}"
        )
        redirect_stderr = stderr_path

    # Create and register process
    # 建立並註冊程序
    execute_process = ExecuteProcess()
    process_manager_instance.process_dict[name] = execute_process

    # Assign redirections if provided
    # 若有設定則指定重定向
    if redirect_stdout:
        execute_process.redirect_stdout = redirect_stdout
    if redirect_stderr:
        execute_process.redirect_stderr = redirect_stderr

    # Start the process
    # 啟動程式
    execute_process.start_process(program)
    return True


def close_program(step: dict, enable_logging: bool = False) -> bool:
    """
    Close a program using process_manager_instance.
    使用 process_manager_instance 關閉程式。

    Args:
        step (dict): Dictionary containing 'close_program'.
                     包含 'close_program' 的字典。
        enable_logging (bool): Whether to enable logging. 是否啟用日誌紀錄。

    Returns:
        bool: True if success, False otherwise.
              成功回傳 True，失敗回傳 False。
    """
    program = step.get("close_program")
    if not isinstance(program, str):
        step_log_check(
            enable_logging=enable_logging,
            logger=test_pioneer_logger,
            level="error",
            message=f"The 'close_program' parameter must be str, got: {program}"
        )
        return False

    step_log_check(
        enable_logging=enable_logging,
        logger=test_pioneer_logger,
        level="info",
        message=f"Close program: {program}"
    )

    # Close the process
    # 關閉程式
    process_manager_instance.close_process(program)
    return True